package org.refresh.tests;

import org.refresh.core.RefreshEngine;
import org.refresh.core.conf.ProblemParser;
import org.refresh.core.conf.RefreshProblem;
import org.refresh.injectors.ConfigurationBinder;
import org.refresh.probe.Target;

import junit.framework.TestCase;

/******************************************************************************
 * Copyright (c) 2007 Jules White.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jules White - initial API and implementation 
 ****************************************************************************/
public class EngineTest extends TestCase {

	public void testEngineNoProbesWithoutTracing(){
		String problemdef = "\r\n" + 
				"A {\r\n" + 
				"SubFeatures:D,E;\r\n" + 
				"Required:1;\n"+
				"Requires:F;\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"C {\r\n" + 
				" Excludes:F;\r\n" + 
				" Set:A.size=3;\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"F {\r\n" + 
				" Set:A.size=4;\r\n" + 
				" Set:A.attrs=[{id=foo;uri=www.foo.com/F}];\r\n" + 
				" Requires:G;\n"+
				"}\r\n" + 
				"\r\n" + 
				"D {\r\n" + 
				" Set:A.exits=[D];\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"E {\r\n" + 
				" Select:[1..2],F,G;\r\n" + 
				" Set:A.exits=[E];\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"G {\r\n" + 
				" Set:A.attrs=[{id=foo;uri=www.foo.com/G}];\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"";
		
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(problemdef);
		Target target = Target.loadFrom("LocalHost {}");
		problem.getTargetItems().add(target);
		RefreshEngine engine = new RefreshEngine(problem);
		engine.setEnableTracing(false);
		engine.setStripSpaces(true);
		engine.run();
		
		String conf = "\r\n" + 
				"<!--feature[A](replace[@size=${size}]){-->\r\n" + 
				"<a id=\"1\" size=\"2\">\r\n" + 
				"<!--}-->\r\n" + 
				"\r\n" + 
				" <!--feature[A](for-each[list=attrs]){\r\n" + 
				"  <attr id=\"${id}\" uri=\"${uri}\"/>\r\n" + 
				" }-->\r\n" + 
				"\r\n" + 
				"</a>";
		
		String expected = "<a id=\"1\" size=\"4\"><attr id=\"foo\" uri=\"www.foo.com/F\"/><attr id=\"foo\" uri=\"www.foo.com/G\"/></a>";
		
		ConfigurationBinder binder = engine.getBinder();
		
		String result = binder.bind(conf);
		
		result = result.replaceAll("\\n", "");
		result = result.replaceAll("[\\s]+", "");
		expected = expected.replaceAll("\\n", "");
		expected = expected.replaceAll("[\\s]+", "");
		
		result = result.replaceAll("\\n", "");
		result = result.replaceAll("[\\s]+", "");
		assertEquals(expected, result);
	}
	
	public void testEngineNoProbesWithTracing(){
		String problemdef = "\r\n" + 
		"A {\r\n" + 
		"SubFeatures:D,E;\r\n" + 
		"Required:1;\n"+
		"Requires:F;\r\n" + 
		"}\r\n" + 
		"\r\n" + 
		"C {\r\n" + 
		" Excludes:F;\r\n" + 
		" Set:A.size=3;\r\n" + 
		"}\r\n" + 
		"\r\n" + 
		"F {\r\n" + 
		" Set:A.size=4;\r\n" + 
		" Set:A.attrs=[{id=foo;uri=www.foo.com/F}];\r\n" + 
		" Requires:G;\n"+
		"}\r\n" + 
		"\r\n" + 
		"D {\r\n" + 
		" Set:A.exits=[D];\r\n" + 
		"}\r\n" + 
		"\r\n" + 
		"E {\r\n" + 
		" Select:[1..2],F,G;\r\n" + 
		" Set:A.exits=[E];\r\n" + 
		"}\r\n" + 
		"\r\n" + 
		"G {\r\n" + 
		" Set:A.attrs=[{id=foo;uri=www.foo.com/G}];\r\n" + 
		"}\r\n" + 
		"\r\n" + 
		"";
		
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(problemdef);
		Target target = Target.loadFrom("LocalHost {}");
		problem.getTargetItems().add(target);
		RefreshEngine engine = new RefreshEngine(problem);
		engine.run();
		
		String conf = "\r\n" + 
				"<!--feature[A](replace[@size=${size}]){-->\r\n" + 
				"<a id=\"1\" size=\"2\">\r\n" + 
				"<!--}-->\r\n" + 
				"\r\n" + 
				" <!--feature[A](for-each[list=attrs]){\r\n" + 
				"  <attr id=\"${id}\" uri=\"${uri}\"/>\r\n" + 
				" }-->\r\n" + 
				"\r\n" + 
				"</a>";
		
		ConfigurationBinder binder = engine.getBinder();
		
		String expected = "<!--\r\n" + 
				"This configuration binding was produced by the annotation:\r\n" + 
				"##########################################################\r\n" + 
				"  Required Feature:A\r\n" + 
				"  Head            :\r\n" + 
				"                  |feature[A](replace[@size=${size}]){\r\n" + 
				"  Tail            :\r\n" + 
				"                  |}\r\n" + 
				"  Template        :\r\n" + 
				"                  |\r\n" + 
				"                  |<a id=\"1\" size=\"2\">\r\n" + 
				"                  |\r\n" + 
				"  Bindings:\r\n" + 
				"    key:\"exits\" is bound to the list:\r\n" + 
				"     [\r\n" + 
				"      item:\"D\" bound by the feature(s):D\r\n" + 
				"      item:\"E\" bound by the feature(s):E\r\n" + 
				"     ]\r\n" + 
				"    key:\"attrs\" is bound to the list:\r\n" + 
				"     [\r\n" + 
				"      item:\"{id=foo, uri=www.foo.com/F}\" bound by the feature(s):F\r\n" + 
				"      item:\"{id=foo, uri=www.foo.com/G}\" bound by the feature(s):G\r\n" + 
				"     ]\r\n" + 
				"    key:size bound by the feature(s):[F] to:\"4\"\r\n" + 
				"##########################################################\r\n" + 
				"-->\r\n" + 
				"\r\n" + 
				"<a id=\"1\" size=\"4\">\r\n" + 
				"\r\n" + 
				"\r\n" + 
				" <!--\r\n" + 
				"This configuration binding was produced by the annotation:\r\n" + 
				"##########################################################\r\n" + 
				"  Required Feature:A\r\n" + 
				"  Head            :\r\n" + 
				"                  |feature[A](for-each[list=attrs]){\r\n" + 
				"                  |  <attr id=\"${id}\" uri=\"${uri}\"/>\r\n" + 
				"                  | }\r\n" + 
				"  Template        :\r\n" + 
				"                  |<attr id=\"${id}\" uri=\"${uri}\"/>\r\n" + 
				"                  | \r\n" + 
				"  Bindings:\r\n" + 
				"    key:\"exits\" is bound to the list:\r\n" + 
				"     [\r\n" + 
				"      item:\"D\" bound by the feature(s):D\r\n" + 
				"      item:\"E\" bound by the feature(s):E\r\n" + 
				"     ]\r\n" + 
				"    key:\"attrs\" is bound to the list:\r\n" + 
				"     [\r\n" + 
				"      item:\"{id=foo, uri=www.foo.com/F}\" bound by the feature(s):F\r\n" + 
				"      item:\"{id=foo, uri=www.foo.com/G}\" bound by the feature(s):G\r\n" + 
				"     ]\r\n" + 
				"    key:size bound by the feature(s):[F] to:\"4\"\r\n" + 
				"##########################################################\r\n" + 
				"-->\r\n" + 
				"<attr id=\"foo\" uri=\"www.foo.com/F\"/>\r\n" + 
				" <attr id=\"foo\" uri=\"www.foo.com/G\"/>\r\n" + 
				" \r\n" + 
				"\r\n" + 
				"</a>";
		
		
		expected = expected.replaceAll("\\n", "");
		expected = expected.replaceAll("[\\s]+", "");
		String result = binder.bind(conf);
		
		result = result.replaceAll("\\n", "");
		result = result.replaceAll("[\\s]+", "");
		assertEquals(expected, result);
	}
}
