package org.refresh.tests;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.refresh.injectors.CommentBasedInjector;
import org.refresh.injectors.Injector;
import org.refresh.injectors.JavaInjector;
import org.refresh.injectors.AbstractAnnotationBasedInjector.UnknownAnnotationTypeException;
import org.refresh.injectors.annotations.ForEachAnnotation;
import org.refresh.injectors.annotations.InjectionException;
import org.refresh.injectors.annotations.InsertAnnotation;
import org.refresh.injectors.annotations.XMLReplaceAnnotation;
import org.refresh.util.ArgumentsFormatException;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
public class JavaInjectorTest extends TestCase {

	public void enable(String feat, Map emap){
		ArrayList l = new ArrayList();
		l.add(Boolean.TRUE);
		emap.put(feat, l);
	}
	
	public void testInsert(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](insert){private int size = 3;}**/}";
		CommentBasedInjector injector = new JavaInjector();
		injector.addAnnotationHandler(new InsertAnnotation());
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		String expected = "public class MyClass {private int size = 3;}";
		assertEquals(expected,result);
		
	}
	
	
	public void testReplace(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](replace[foo=1]){private int size = ${foo};}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		String expected = "public class MyClass {private int size = 1;}";
		
		assertEquals(expected,result);
		
		foobinds.put("size", "5");
		bindings.put("foo", foobinds);
		conf = "public class MyClass {/**feature[foo](replace[foo=${size}]){**/private int size = ${foo};/**}**/}";
		injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		
		result = injector.inject(enabled,bindings, conf);
		expected = "public class MyClass {private int size = 5;}";
		
		assertEquals(expected,result);
		
	}
	
	
	public void testReplaceUsingWith(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](replace){**/private int size = ${foo};/**}with{private Integer size = 8;}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		String expected = "public class MyClass {private Integer size = 8;}";
		
		assertEquals(expected,result);
	}
	
	
	
	public void testForEachWithSimpleList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		mylist.add("foo");
		mylist.add("bar");
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](for-each[list=mylist]){**/private String ${value} = \"\";/**}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		String expected = "public class MyClass {private String foo = \"\";private String bar = \"\";}";
		String result = injector.inject(enabled,bindings, conf);
		assertEquals(expected,result);
	}
	
	public void testForEachWithMapList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		Map attrs = new HashMap();
		attrs.put("id", "foo");
		attrs.put("size","1");
		mylist.add(attrs);
		attrs = new HashMap();
		attrs.put("id", "bar");
		attrs.put("size", "2");
		mylist.add(attrs);
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](for-each[list=mylist]){**/private int ${id} = ${size};/**}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		String expected = "public class MyClass {private int foo = 1;private int bar = 2;}";
		String result = injector.inject(enabled,bindings, conf);
		assertEquals(expected,result);

	}
	
}
