package org.refresh.tests;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.refresh.injectors.CommentBasedInjector;
import org.refresh.injectors.JavaInjector;
import org.refresh.injectors.PrologInjector;
import org.refresh.injectors.annotations.InsertAnnotation;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
public class PrologInjectorTest extends TestCase {

	public void enable(String feat, Map emap){
		ArrayList l = new ArrayList();
		l.add(Boolean.TRUE);
		emap.put(feat, l);
	}
	
	public void testInsert(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "self_name(%feature[foo](insert){some_id}%\n,N).";
		CommentBasedInjector injector = PrologInjector.createDefaultInjector();
		injector.addAnnotationHandler(new InsertAnnotation());
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		//String expected = "public class MyClass {private int size = 3;}";
		//assertEquals(expected,result);
		System.out.println(result);
		
	}
	
	public void testInsertCSaw(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("A", enabled);
		enable("B", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "defines RootAspect,AddCaches#feature[*](insert){,${feature}AddCache}#; \r\n" + 
				" \r\n" + 
				"\r\n" + 
				"aspect RootAspect()\r\n" + 
				"{\r\n" + 
				"rootFolder().models()->AddCaches();\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"\r\n" + 
				"strategy AddCaches() \r\n" + 
				" \r\n" + 
				"{ \r\n" + 
				"    declare parentModel : model;\r\n" + 
				"  \r\n" + 
				"    parentModel := self; \r\n" + 
				"\r\n" + 
				"    #feature[*](insert){# \r\n" + 
				"    parentModel.atoms(\"Component\")->select(m|m.name() == \"${feature}\")->${feature}AddCache ( ); \r\n" + 
				"    #}#\r\n" + 
				"\r\n" + 
				"}\r\n" + 
				"\r\n" + 
				"\r\n" + 
				"#feature[*](insert){# \r\n" + 
				"strategy ${feature}AddCache( ) { \r\n" + 
				" \r\n" + 
				"    declare parentModel : model; \r\n" + 
				"\r\n" + 
				"    declare component, cache : atom;\r\n" + 
				" \r\n" + 
				"    parentModel := parent(); \r\n" + 
				"\r\n" + 
				"    component := self;\r\n" + 
				"\r\n" + 
				"    cache := parentModel.addAtom(\"Cache\", \"CacheFor${feature}\"); \r\n" + 
				" \r\n" + 
				"    parentModel.addConnection(\"CacheInstallation\",cache,component);\r\n" + 
				"    \r\n" + 
				"}\r\n" + 
				"#}# ";
		CommentBasedInjector injector = new CommentBasedInjector("#","#");
		injector.addAnnotationHandler(new InsertAnnotation());
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		//String expected = "public class MyClass {private int size = 3;}";
		//assertEquals(expected,result);
		System.out.println(result);
		
	}
	
	
	
	public void testReplace(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](replace[foo=1]){private int size = ${foo};}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		String expected = "public class MyClass {private int size = 1;}";
		
		assertEquals(expected,result);
		
		foobinds.put("size", "5");
		bindings.put("foo", foobinds);
		conf = "public class MyClass {/**feature[foo](replace[foo=${size}]){**/private int size = ${foo};/**}**/}";
		injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		
		result = injector.inject(enabled,bindings, conf);
		expected = "public class MyClass {private int size = 5;}";
		
		assertEquals(expected,result);
		
	}
	
	
	public void testReplaceUsingWith(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](replace){**/private int size = ${foo};/**}with{private Integer size = 8;}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		
		String result = injector.inject(enabled,bindings, conf);
		String expected = "public class MyClass {private Integer size = 8;}";
		
		assertEquals(expected,result);
	}
	
	
	
	public void testForEachWithSimpleList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		mylist.add("foo");
		mylist.add("bar");
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](for-each[list=mylist]){**/private String ${value} = \"\";/**}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		String expected = "public class MyClass {private String foo = \"\";private String bar = \"\";}";
		String result = injector.inject(enabled,bindings, conf);
		assertEquals(expected,result);
	}
	
	public void testForEachWithMapList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		Map attrs = new HashMap();
		attrs.put("id", "foo");
		attrs.put("size","1");
		mylist.add(attrs);
		attrs = new HashMap();
		attrs.put("id", "bar");
		attrs.put("size", "2");
		mylist.add(attrs);
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "public class MyClass {/**feature[foo](for-each[list=mylist]){**/private int ${id} = ${size};/**}**/}";
		CommentBasedInjector injector = JavaInjector.createDefaultInjector();
		injector.setEnableTracing(false);
		String expected = "public class MyClass {private int foo = 1;private int bar = 2;}";
		String result = injector.inject(enabled,bindings, conf);
		assertEquals(expected,result);

	}
	
}
