package org.refresh.tests;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.refresh.injectors.Injector;
import org.refresh.injectors.XMLInjector;
import org.refresh.injectors.AbstractAnnotationBasedInjector.UnknownAnnotationTypeException;
import org.refresh.injectors.annotations.ForEachAnnotation;
import org.refresh.injectors.annotations.InjectionException;
import org.refresh.injectors.annotations.InsertAnnotation;
import org.refresh.injectors.annotations.XMLReplaceAnnotation;
import org.refresh.util.ArgumentsFormatException;
import org.refresh.util.ParsingUtil;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
public class XmlInjectorTest extends TestCase {

	public void enable(String feat, Map emap){
		ArrayList l = new ArrayList();
		l.add(Boolean.TRUE);
		emap.put(feat, l);
	}
	
	public void testInsert(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](insert){<b></b>}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new InsertAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b></b></a>",injector.inject(enabled,bindings, conf));
		
//		HashMap<String, Map<String,List<String>>> traces = new HashMap<String, Map<String,List<String>>>();
//		HashMap<String, List<String>> footrace = new HashMap<String, List<String>>();
//		List<String> binders = new ArrayList<String>();
//		binders.add("foo.child");
//		footrace.put("id", binders);
//		traces.put("foo", footrace);
//		injector.setEnableTracing(true);
//		injector.setBindingsTraceInformation(traces);
//		System.out.println(injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceAttribute(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](replace[@id=${id}]){--><b id=\"wrong\"></b><!--}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"some_id\"></b></a>",injector.inject(enabled,bindings, conf));
		
		HashMap<String, Map<String,List<String>>> traces = new HashMap<String, Map<String,List<String>>>();
		HashMap<String, List<String>> footrace = new HashMap<String, List<String>>();
		List<String> binders = new ArrayList<String>();
		binders.add("foo.child");
		footrace.put("id", binders);
		traces.put("foo", footrace);
		injector.setEnableTracing(true);
		injector.setBindingsTraceInformation(traces);
		System.out.println(injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceValue(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](replace[#value=foobar]){--><b id=\"a\">asdfa</b><!--}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"a\">foobar</b></a>",injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceValueUsingWith(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](replace[#value=${with}]){--><b id=\"a\">asdfa</b><!--} with {<c><d/></c>}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"a\"><c><d/></c></b></a>",injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceWith(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](replace){--><b id=\"a\">asdfa</b><!--} with {<c><d/></c>}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><c><d/></c></a>",injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceMix(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](replace[#value=${with}; @id=${id}; @size=2]){--><b id=\"a\" size=\"3\">asdfa</b><!--} with {<c><d/></c>}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"some_id\" size=\"2\"><c><d/></c></b></a>",injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceMixWithMultipleSources(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		enable("bar", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		
		HashMap<String, Object> barbinds = new HashMap<String, Object>();
		barbinds.put("foo.ior","234231432434");
		bindings.put("bar",barbinds);
		HashMap attrs = new HashMap();
		attrs.put("a", "bb");
		attrs.put("c", "dd");
		barbinds.put("foo.attrs",attrs);
		
		String conf = "<a><!--feature[foo](replace[#value=${with}; @id=${id}; @ior=${ior}; @size=2]){--><b id=\"a\" ior=\"11111111\" size=\"3\">asdfa</b><!--} with {<c><d/></c>}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"some_id\" ior=\"234231432434\" size=\"2\"><c><d/></c></b></a>",injector.inject(enabled,bindings, conf));
		
//		injector.setEnableTracing(true);
//		System.out.println(injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceMixWithWildcard(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		enable("bar", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		
		HashMap<String, Object> barbinds = new HashMap<String, Object>();
		barbinds.put("id","234231432434");
		bindings.put("bar",barbinds);
		
		HashMap attrs = new HashMap();
		attrs.put("a", "bb");
		attrs.put("c", "dd");
		barbinds.put("foo.attrs",attrs);
		
		String conf = "<a><!--feature[*](replace[#value=${with}; @id=${id}; @size=2]){--><b id=\"a\" ior=\"11111111\" size=\"3\">asdfa</b><!--} with {<c><d/></c>}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		String result = injector.inject(enabled,bindings, conf);
		
		String expected = "<a><b id=\"some_id\" ior=\"11111111\" size=\"2\"><c><d/></c></b><b id=\"234231432434\" ior=\"11111111\" size=\"2\"><c><d/></c></b></a>";
		assertEquals(expected,result);
		
//		injector.setEnableTracing(true);
//		System.out.println(injector.inject(enabled,bindings, conf));
	}
	
	public void testReplaceMixWithWildcardAndMultiDirectives(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		enable("bar", enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		
		foobinds.put("id", "some_id");
		bindings.put("foo", foobinds);
		
		HashMap<String, Object> barbinds = new HashMap<String, Object>();
		barbinds.put("id","234231432434");
		bindings.put("bar",barbinds);
		
		HashMap attrs = new HashMap();
		attrs.put("a", "bb");
		attrs.put("c", "dd");
		barbinds.put("foo.attrs",attrs);
		
		String conf = "<a><!--feature[*](replace[#value=${with}; @id=${id}; @size=2]){--><b id=\"a\" ior=\"11111111\" size=\"3\">asdfa</b><!--} with {<c><d/></c>}--><!--feature[foo](insert){<q/>}--></a>";
		XMLInjector injector = XMLInjector.createDefaultInjector();
		injector.addAnnotationHandler(new XMLReplaceAnnotation());
		injector.setEnableTracing(false);
		String result = injector.inject(enabled,bindings, conf);
		
		String expected = "<a><b id=\"some_id\" ior=\"11111111\" size=\"2\"><c><d/></c></b><b id=\"234231432434\" ior=\"11111111\" size=\"2\"><c><d/></c></b><q/></a>";
		assertEquals(expected,result);
		
//		injector.setEnableTracing(true);
//		System.out.println(injector.inject(enabled,bindings, conf));
	}
	
	public void testForEachWithSimpleList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		mylist.add("foo");
		mylist.add("bar");
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](for-each[list=mylist]){--><b id=\"${value}\"/><!--}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new ForEachAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"foo\"/><b id=\"bar\"/></a>",injector.inject(enabled,bindings, conf));
	}
	
	public void testForEachWithMapList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		enable("foo",enabled);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		Map attrs = new HashMap();
		attrs.put("id", "foo");
		attrs.put("size","1");
		mylist.add(attrs);
		attrs = new HashMap();
		attrs.put("id", "bar");
		attrs.put("size", "2");
		mylist.add(attrs);
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](for-each[list=mylist]){--><b id=\"${id}\" size=\"${size}\"/><!--}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new ForEachAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b id=\"foo\" size=\"1\"/><b id=\"bar\" size=\"2\"/></a>",injector.inject(enabled,bindings, conf));
		

//		HashMap<String, Map<String,List<String>>> traces = new HashMap<String, Map<String,List<String>>>();
//		HashMap<String, List<String>> footrace = new HashMap<String, List<String>>();
//		List<String> binders = new ArrayList<String>();
//		binders.add("foo.child");
//		binders.add("foo.child2");
//		footrace.put("mylist", binders);
//		traces.put("foo", footrace);
//		injector.setEnableTracing(true);
//		injector.setBindingsTraceInformation(traces);
//		System.out.println(injector.inject(enabled,bindings, conf));
	}
	
	public void testForEachWithMultipleTargetsList(){
		HashMap<String,List> enabled = new HashMap<String,List>();
		ArrayList targets = new ArrayList();
		targets.add(ParsingUtil.parseMap("{ip=192.168.1.1;}"));
		targets.add(ParsingUtil.parseMap("{ip=192.168.1.7;}"));
		enabled.put("foo", targets);
		HashMap<String, Map> bindings = new HashMap<String, Map>();
		HashMap<String, Object> foobinds = new HashMap<String, Object>();
		ArrayList mylist = new ArrayList();
		Map attrs = new HashMap();
		attrs.put("id", "foo");
		attrs.put("size","1");
		mylist.add(attrs);
		attrs = new HashMap();
		attrs.put("id", "bar");
		attrs.put("size", "2");
		mylist.add(attrs);
		foobinds.put("mylist",mylist);
		bindings.put("foo", foobinds);
		String conf = "<a><!--feature[foo](for-each[list=targets]){--><b ip=\"${ip}\"/><!--}--></a>";
		XMLInjector injector = new XMLInjector();
		injector.addAnnotationHandler(new ForEachAnnotation());
		injector.setEnableTracing(false);
		assertEquals("<a><b ip=\"192.168.1.1\"/><b ip=\"192.168.1.7\"/></a>",injector.inject(enabled,bindings, conf));
	}
	
	public void testInvalidAnnotation(){
		String conf = "<a><!--feature[foo](non-existant-annotation[a=b]){--><b></b><!--} with {my_with_stmt}--></a>";
		Injector injector = new XMLInjector();
		try{
			injector.inject(new HashMap<String,List>(), new HashMap<String, Map>(), conf);
			fail("The XMLInjector should have thrown an UnknownAnnotationTypeException");
		}
		catch (InjectionException e) {
			assertEquals(UnknownAnnotationTypeException.class, e.getCause().getClass());
		}
	}
	
	public void testInvalidArgs(){
		String conf = "<a><!--feature[foo](remove[a=]){<b></b>}--></a>";
		Injector injector = new XMLInjector();
		try{
			injector.inject(new HashMap<String,List>(), new HashMap<String, Map>(), conf);
			fail("The XMLInjector should have thrown an ArgumentsFormatException");
		}
		catch (InjectionException e) {
			assertEquals(ArgumentsFormatException.class, e.getCause().getClass());
		}
	}
	
}
